@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
-from django.conf.urls.defaults import *
+from django.conf.urls import patterns, include, url
 from django.core.urlresolvers import reverse
-from django.views.generic.simple import direct_to_template
+from django.views.generic import TemplateView
+
 from codespeed.feeds import LatestEntries
 
-feeds = { 'latest': LatestEntries }
-
+feeds = {'latest': LatestEntries}
 
 urlpatterns = patterns('',
-    (r'^$', direct_to_template, {'template': 'home.html'}),
-    (r'^about/$', direct_to_template, {'template': 'about.html'}),
+    (r'^$', TemplateView.as_view(template_name='home.html')),
+    (r'^about/$', TemplateView.as_view(template_name='about.html')),
     # RSS for reports
     (r'^feeds/(?P<url>.*)/$', LatestEntries()),
 )
@@ -28,5 +28,5 @@ urlpatterns += patterns('codespeed.views',
 urlpatterns += patterns('codespeed.views',
     # URLs for adding results
     (r'^result/add/json/$', 'add_json_results'),
-    (r'^result/add/$',      'add_result'),
+    (r'^result/add/$', 'add_result'),
 )
