@@ -9,27 +9,42 @@ from django.http import (HttpResponse, Http404, HttpResponseNotAllowed,
 from django.shortcuts import get_object_or_404, render_to_response
 from django.template import RequestContext
 from django.core.exceptions import ValidationError
+from django.core.urlresolvers import reverse
 from django.conf import settings
+from django.views.decorators.csrf import csrf_exempt
 
 from codespeed.models import (Environment, Report, Project, Revision, Result,
                               Executable, Benchmark, Branch)
 
 
+logger = logging.getLogger(__name__)
+
+
 def no_environment_error(request):
+    admin_url = reverse('admin:codespeed_environment_changelist')
     return render_to_response('codespeed/nodata.html', {
-        'message': 'You need to configure at least one Environment. Please go to the <a href="../admin/codespeed/environment/">admin interface</a>'
+        'message': ('You need to configure at least one Environment. '
+                    'Please go to the '
+                    '<a href="%s">admin interface</a>' % admin_url)
     }, context_instance=RequestContext(request))
 
+
 def no_default_project_error(request):
+    admin_url = reverse('admin:codespeed_project_changelist')
     return render_to_response('codespeed/nodata.html', {
-        'message': 'You need to configure at least one one Project as default (checked "Track changes" field).<br />Please go to the <a href="../admin/codespeed/project/">admin interface</a>'
+        'message': ('You need to configure at least one one Project as '
+                    'default (checked "Track changes" field).<br />'
+                    'Please go to the '
+                    '<a href="%s">admin interface</a>' % admin_url)
     }, context_instance=RequestContext(request))
 
+
 def no_executables_error(request):
     return render_to_response('codespeed/nodata.html', {
         'message': 'There needs to be at least one executable'
     }, context_instance=RequestContext(request))
 
+
 def no_data_found(request):
     return render_to_response('codespeed/nodata.html', {
         'message': 'No data found'
@@ -37,14 +52,21 @@ def no_data_found(request):
 
 
 def getbaselineexecutables():
-    baseline = [{'key': "none", 'name': "None", 'executable': "none", 'revision': "none"}]
-    revs = Revision.objects.exclude(tag="")
+    baseline = [{
+        'key': "none",
+        'name': "None",
+        'executable': "none",
+        'revision': "none",
+    }]
+    executables = Executable.objects.select_related('project')
+    revs = Revision.objects.exclude(tag="").select_related('branch__project')
     maxlen = 22
     for rev in revs:
-        #add executables that correspond to each tagged revision.
-        for exe in Executable.objects.filter(project=rev.branch.project):
+        # Add executables that correspond to each tagged revision.
+        for exe in [e for e in executables if e.project == rev.branch.project]:
             exestring = str(exe)
-            if len(exestring) > maxlen: exestring = str(exe)[0:maxlen] + "..."
+            if len(exestring) > maxlen:
+                exestring = str(exe)[0:maxlen] + "..."
             name = exestring + " " + rev.tag
             key = str(exe.id) + "+" + str(rev.id)
             baseline.append({
@@ -54,14 +76,15 @@ def getbaselineexecutables():
                 'name': name,
             })
     # move default to first place
-    if hasattr(settings, 'DEF_BASELINE') and settings.DEF_BASELINE != None:
+    if hasattr(settings, 'DEF_BASELINE') and settings.DEF_BASELINE is not None:
         try:
+            exename = settings.DEF_BASELINE['executable']
+            commitid = settings.DEF_BASELINE['revision']
             for base in baseline:
                 if base['key'] == "none":
                     continue
-                exename = settings.DEF_BASELINE['executable']
-                commitid = settings.DEF_BASELINE['revision']
-                if base['executable'].name == exename and base['revision'].commitid == commitid:
+                if (base['executable'].name == exename and
+                        base['revision'].commitid == commitid):
                     baseline.remove(base)
                     baseline.insert(1, base)
                     break
@@ -96,8 +119,8 @@ def get_default_environment(enviros, data, multi=False):
                 break
     # Use settings.py value
     if not defaultenviros and not multi:
-        if hasattr(settings, 'DEF_ENVIRONMENT') and \
-            settings.DEF_ENVIRONMENT != None:
+        if (hasattr(settings, 'DEF_ENVIRONMENT') and
+                settings.DEF_ENVIRONMENT is not None):
             for env in enviros:
                 if settings.DEF_ENVIRONMENT == env.name:
                     defaultenviros.append(env)
@@ -113,12 +136,12 @@ def get_default_environment(enviros, data, multi=False):
 
 def getdefaultexecutable():
     default = None
-    if hasattr(settings, 'DEF_EXECUTABLE') and settings.DEF_EXECUTABLE != None:
+    if hasattr(settings, 'DEF_EXECUTABLE') and settings.DEF_EXECUTABLE is not None:
         try:
             default = Executable.objects.get(name=settings.DEF_EXECUTABLE)
         except Executable.DoesNotExist:
             pass
-    if default == None:
+    if default is None:
         execquery = Executable.objects.filter(project__track=True)
         if len(execquery):
             default = execquery[0]
@@ -136,7 +159,7 @@ def getcomparisonexes():
         maxlen = 20
         # add all tagged revs for any project
         for exe in baselines:
-            if exe['key'] != "none" and exe['executable'].project == proj:
+            if exe['key'] is not "none" and exe['executable'].project == proj:
                 executablekeys.append(exe['key'])
                 executables.append(exe)
 
@@ -176,28 +199,31 @@ def getcomparisondata(request):
     data = request.GET
 
     executables, exekeys = getcomparisonexes()
+    benchmarks = Benchmark.objects.all()
+    environments = Environment.objects.all()
 
     compdata = {}
     compdata['error'] = "Unknown error"
     for proj in executables:
         for exe in executables[proj]:
             compdata[exe['key']] = {}
-            for env in Environment.objects.all():
+            for env in environments:
                 compdata[exe['key']][env.id] = {}
-                for bench in Benchmark.objects.all().order_by('name'):
-                    try:
-                        value = Result.objects.get(
-                            environment=env,
-                            executable=exe['executable'],
-                            revision=exe['revision'],
-                            benchmark=bench
-                        ).value
-                    except Result.DoesNotExist:
-                        value = None
-                    compdata[exe['key']][env.id][bench.id] = value
+
+                # Load all results for this env/executable/revision in a dict
+                # for fast lookup
+                results = dict(Result.objects.filter(
+                    environment=env,
+                    executable=exe['executable'],
+                    revision=exe['revision'],
+                ).values_list('benchmark', 'value'))
+
+                for bench in benchmarks:
+                    compdata[exe['key']][env.id][bench.id] = results.get(bench.id, None)
+
     compdata['error'] = "None"
 
-    return HttpResponse(json.dumps( compdata ))
+    return HttpResponse(json.dumps(compdata))
 
 
 def comparison(request):
@@ -226,8 +252,7 @@ def comparison(request):
                 continue
             if i in exekeys:
                 checkedexecutables.append(i)
-    elif hasattr(settings, 'COMP_EXECUTABLES') and\
-        settings.COMP_EXECUTABLES:
+    elif hasattr(settings, 'COMP_EXECUTABLES') and settings.COMP_EXECUTABLES:
         for exe, rev in settings.COMP_EXECUTABLES:
             try:
                 exe = Executable.objects.get(name=exe)
@@ -264,25 +289,31 @@ def comparison(request):
             benchmark_type="C"
         ).filter(units_title=unit)
         units = benchmarks[unit][0].units
-        lessisbetter = benchmarks[unit][0].lessisbetter and ' (less is better)' or ' (more is better)'
-        bench_units[unit] = [[b.id for b in benchmarks[unit]], lessisbetter, units]
+        lessisbetter = (benchmarks[unit][0].lessisbetter and
+                        ' (less is better)' or ' (more is better)')
+        bench_units[unit] = [
+            [b.id for b in benchmarks[unit]], lessisbetter, units
+        ]
     checkedbenchmarks = []
     if 'ben' in data:
         checkedbenchmarks = []
         for i in data['ben'].split(","):
-            if not i: continue
+            if not i:
+                continue
             try:
                 checkedbenchmarks.append(Benchmark.objects.get(id=int(i)))
             except Benchmark.DoesNotExist:
                 pass
     if not checkedbenchmarks:
         # Only include benchmarks marked as cross-project
-        checkedbenchmarks = Benchmark.objects.filter(benchmark_type="C")
+        checkedbenchmarks = Benchmark.objects.filter(
+            benchmark_type="C", default_on_comparison=True)
 
     charts = ['normal bars', 'stacked bars', 'relative bars']
     # Don't show relative charts as an option if there is only one executable
     # Relative charts need normalization
-    if len(executables) == 1: charts.remove('relative bars')
+    if len(executables) == 1:
+        charts.remove('relative bars')
 
     selectedchart = charts[0]
     if 'chart' in data and data['chart'] in charts:
@@ -296,12 +327,12 @@ def comparison(request):
     elif 'bas' in data:
         # bas is present but is none
         pass
-    elif len(exekeys) > 1 and hasattr(settings, 'NORMALIZATION') and\
-        settings.NORMALIZATION:
+    elif (len(exekeys) > 1 and hasattr(settings, 'NORMALIZATION') and
+            settings.NORMALIZATION):
         try:
             # TODO: Avoid calling twice getbaselineexecutables
             selectedbaseline = getbaselineexecutables()[1]['key']
-            # Uncheck exe used for normalization 
+            # Uncheck exe used for normalization
             try:
                 checkedexecutables.remove(selectedbaseline)
             except ValueError:
@@ -310,8 +341,9 @@ def comparison(request):
             pass  # Keep "none" as default baseline
 
     selecteddirection = False
-    if 'hor' in data and data['hor'] == "true" or\
-        hasattr(settings, 'CHART_ORIENTATION') and settings.CHART_ORIENTATION == 'horizontal':
+    if ('hor' in data and data['hor'] == "true" or
+        hasattr(settings, 'CHART_ORIENTATION') and
+            settings.CHART_ORIENTATION == 'horizontal'):
         selecteddirection = True
 
     return render_to_response('codespeed/comparison.html', {
@@ -339,7 +371,7 @@ def gettimelinedata(request):
     executables = data.get('exe', "").split(",")
     if not filter(None, executables):
         timeline_list['error'] = "No executables selected"
-        return HttpResponse(json.dumps( timeline_list ))
+        return HttpResponse(json.dumps(timeline_list))
     environment = None
     try:
         environment = get_object_or_404(Environment, id=data.get('env'))
@@ -378,37 +410,40 @@ def gettimelinedata(request):
         trunks = []
         if Branch.objects.filter(name=settings.DEF_BRANCH):
             trunks.append(settings.DEF_BRANCH)
-        #for branch in data2.get('bran', '').split(','): #-- For now, we'll only work with trunk branches
+        # For now, we'll only work with trunk branches
         append = False
         for branch in trunks:
             append = False
             timeline['branches'][branch] = {}
             for executable in executables:
                 resultquery = Result.objects.filter(
-                        benchmark=bench
-                    ).filter(
-                        environment=environment
-                    ).filter(
-                        executable=executable
-                    ).filter(
-                        revision__branch__name=branch
-                    ).select_related(
-                        "revision"
-                    ).order_by('-revision__date')[:number_of_revs]
+                    benchmark=bench
+                ).filter(
+                    environment=environment
+                ).filter(
+                    executable=executable
+                ).filter(
+                    revision__branch__name=branch
+                ).select_related(
+                    "revision"
+                ).order_by('-revision__date')[:number_of_revs]
                 if not len(resultquery):
                     continue
 
                 results = []
                 for res in resultquery:
                     std_dev = ""
-                    if res.std_dev != None:
+                    if res.std_dev is not None:
                         std_dev = res.std_dev
                     results.append(
-                        [str(res.revision.date), res.value, std_dev, res.revision.get_short_commitid(), branch]
+                        [
+                            res.revision.date.isoformat(), res.value, std_dev,
+                            res.revision.get_short_commitid(), branch
+                        ]
                     )
                 timeline['branches'][branch][executable] = results
                 append = True
-            if baselinerev != None and append:
+            if baselinerev is not None and append:
                 try:
                     baselinevalue = Result.objects.get(
                         executable=baselineexe,
@@ -419,13 +454,14 @@ def gettimelinedata(request):
                 except Result.DoesNotExist:
                     timeline['baseline'] = "None"
                 else:
-                    # determine start and end revision (x axis) from longest data series
+                    # determine start and end revision (x axis)
+                    # from longest data series
                     results = []
                     for exe in timeline['branches'][branch]:
                         if len(timeline['branches'][branch][exe]) > len(results):
                             results = timeline['branches'][branch][exe]
                     end = results[0][0]
-                    start = results[len(results)-1][0]
+                    start = results[len(results) - 1][0]
                     timeline['baseline'] = [
                         [str(start), baselinevalue],
                         [str(end), baselinevalue]
@@ -436,7 +472,7 @@ def gettimelinedata(request):
     if not len(timeline_list['timelines']) and data['ben'] != 'show_none':
         response = 'No data found for the selected options'
         timeline_list['error'] = response
-    return HttpResponse(json.dumps( timeline_list ))
+    return HttpResponse(json.dumps(timeline_list))
 
 
 def timeline(request):
@@ -461,7 +497,8 @@ def timeline(request):
     checkedexecutables = []
     if 'exe' in data:
         for i in data['exe'].split(","):
-            if not i: continue
+            if not i:
+                continue
             try:
                 checkedexecutables.append(Executable.objects.get(id=int(i)))
             except Executable.DoesNotExist:
@@ -509,13 +546,13 @@ def timeline(request):
         return no_data_found(request)
     elif len(benchmarks) == 1:
         defaultbenchmark = benchmarks[0]
-    elif hasattr(settings, 'DEF_BENCHMARK') and settings.DEF_BENCHMARK != None:
+    elif hasattr(settings, 'DEF_BENCHMARK') and settings.DEF_BENCHMARK is not None:
         if settings.DEF_BENCHMARK in ['grid', 'show_none']:
             defaultbenchmark = settings.DEF_BENCHMARK
         else:
             try:
                 defaultbenchmark = Benchmark.objects.get(
-                                                name=settings.DEF_BENCHMARK)
+                    name=settings.DEF_BENCHMARK)
             except Benchmark.DoesNotExist:
                 pass
     elif len(benchmarks) >= grid_limit:
@@ -526,7 +563,7 @@ def timeline(request):
             defaultbenchmark = data['ben']
         else:
             defaultbenchmark = get_object_or_404(Benchmark, name=data['ben'])
-    
+
     if 'equid' in data:
         defaultequid = data['equid']
     else:
@@ -549,7 +586,7 @@ def timeline(request):
         'environments': enviros,
         'branch_list': branch_list,
         'defaultbranch': defaultbranch,
-        'defaultequid' : defaultequid
+        'defaultequid': defaultequid
     }, context_instance=RequestContext(request))
 
 
@@ -569,7 +606,10 @@ def getchangestable(request):
     tablelist = report.get_changes_table(trendconfig)
 
     if not len(tablelist):
-        return HttpResponse('<table id="results" class="tablesorter" style="height: 232px;"></table><p class="errormessage">No results for this parameters</p>')
+        return HttpResponse('<table id="results" class="tablesorter" '
+                            'style="height: 232px;"></table>'
+                            '<p class="errormessage">No results for this '
+                            'parameters</p>')
 
     return render_to_response('codespeed/changes_table.html', {
         'tablelist': tablelist,
@@ -581,17 +621,18 @@ def getchangestable(request):
 
 
 def changes(request):
-    if request.method != 'GET': return HttpResponseNotAllowed('GET')
+    if request.method != 'GET':
+        return HttpResponseNotAllowed('GET')
     data = request.GET
 
     # Configuration of default parameters
     defaultchangethres = 3.0
     defaulttrendthres = 4.0
-    if hasattr(settings, 'CHANGE_THRESHOLD') and \
-                                        settings.CHANGE_THRESHOLD != None:
+    if (hasattr(settings, 'CHANGE_THRESHOLD') and
+            settings.CHANGE_THRESHOLD is not None):
         defaultchangethres = settings.CHANGE_THRESHOLD
-    if hasattr(settings, 'TREND_THRESHOLD') and \
-                                        settings.TREND_THRESHOLD != None:
+    if (hasattr(settings, 'TREND_THRESHOLD') and
+            settings.TREND_THRESHOLD is not None):
         defaulttrendthres = settings.TREND_THRESHOLD
 
     defaulttrend = 10
@@ -665,6 +706,12 @@ def changes(request):
             projectmatrix[e.id] = e.project.name
     projectmatrix = json.dumps(projectmatrix)
 
+    for project, revisions in revisionlists.items():
+        revisionlists[project] = [
+            (unicode(rev), rev.commitid) for rev in revisions
+        ]
+    revisionlists = json.dumps(revisionlists)
+
     return render_to_response('codespeed/changes.html', {
         'defaultenvironment': defaultenv,
         'defaultexecutable': defaultexecutable,
@@ -675,7 +722,7 @@ def changes(request):
         'environments': enviros,
         'executables': executables,
         'projectmatrix': projectmatrix,
-        'revisionboxes': revisionlists,
+        'revisionlists': revisionlists,
         'trends': trends,
     }, context_instance=RequestContext(request))
 
@@ -694,7 +741,14 @@ def reports(request):
 def displaylogs(request):
     rev = get_object_or_404(Revision, pk=request.GET.get('revisionid'))
     logs = []
-    logs.append(rev)
+    logs.append(
+        {
+            'date': str(rev.date), 'author': rev.author,
+            'author_email': '', 'message': rev.message,
+            'short_commit_id': rev.get_short_commitid(),
+            'commitid': rev.commitid
+        }
+    )
     error = False
     try:
         startrev = Revision.objects.filter(
@@ -711,17 +765,26 @@ def displaylogs(request):
                 if remotelogs[0]['error']:
                     error = remotelogs[0]['message']
             except KeyError:
-                pass#no errors
+                pass  # no errors
             logs = remotelogs
         else:
             error = 'no logs found'
-    except StandardError, e:
-        logging.error("Unhandled exception displaying logs for %s: %s", rev, e, exc_info=True)
+    except (StandardError, RuntimeError) as e:
+        logger.error(
+            "Unhandled exception displaying logs for %s: %s",
+            rev, e, exc_info=True)
         error = repr(e)
 
-    return render_to_response('codespeed/changes_logs.html',
-                                {'error': error, 'logs': logs },
-                                context_instance=RequestContext(request))
+    # add commit browsing url to logs
+    project = rev.branch.project
+    for log in logs:
+        log['commit_browse_url'] = project.commit_browsing_url.format(**log)
+
+    return render_to_response(
+        'codespeed/changes_logs.html',
+        {'error': error, 'logs': logs,
+         'show_email_address': settings.SHOW_AUTHOR_EMAIL_ADDRESS},
+        context_instance=RequestContext(request))
 
 
 def getcommitlogs(rev, startrev, update=False):
@@ -737,8 +800,8 @@ def getcommitlogs(rev, startrev, update=False):
         from github import getlogs, updaterepo
     else:
         if rev.branch.project.repo_type not in ("N", ""):
-            logging.warning("Don't know how to retrieve logs from %s project",
-                                rev.branch.project.get_repo_type_display())
+            logger.warning("Don't know how to retrieve logs from %s project",
+                           rev.branch.project.get_repo_type_display())
         return logs
 
     if update:
@@ -758,8 +821,8 @@ def saverevisioninfo(rev):
 
     if log:
         log = log[0]
-        rev.author  = log['author']
-        rev.date    = log['date']
+        rev.author = log['author']
+        rev.date = log['date']
         rev.message = log['message']
 
 
@@ -782,7 +845,7 @@ def validate_result(item):
     ]
 
     response = {}
-    error    = True
+    error = True
     for key in mandatory_data:
         if not key in item:
             return 'Key "' + key + '" missing from request', error
@@ -805,17 +868,18 @@ def create_report_if_enough_data(rev, exe, e):
     ).order_by('-date')[:2]
     if len(last_revs) > 1:
         current_results = rev.results.filter(executable=exe, environment=e)
-        last_results = last_revs[1].results.filter(executable=exe,environment=e)
+        last_results = last_revs[1].results.filter(
+            executable=exe, environment=e)
         # If there is are at least as many results as in the last revision,
         # create new report
         if len(current_results) >= len(last_results):
-            logging.debug("create_report_if_enough_data: About to create new report")
+            logger.debug("create_report_if_enough_data: About to create new report")
             report, created = Report.objects.get_or_create(
                 executable=exe, environment=e, revision=rev
             )
             report.full_clean()
             report.save()
-            logging.debug("create_report_if_enough_data: Created new report.")
+            logger.debug("create_report_if_enough_data: Created new report.")
 
 
 def save_result(data):
@@ -828,8 +892,21 @@ def save_result(data):
 
     p, created = Project.objects.get_or_create(name=data["project"])
     branch, created = Branch.objects.get_or_create(name=data["branch"],
-                                                    project=p)
+                                                   project=p)
     b, created = Benchmark.objects.get_or_create(name=data["benchmark"])
+
+    if created:
+        if "description" in data:
+            b.description = data["description"]
+        if "units" in data:
+            b.units = data["units"]
+        if "units_title" in data:
+            b.units_title = data["units_title"]
+        if "lessisbetter" in data:
+            b.lessisbetter = data["lessisbetter"]
+        b.full_clean()
+        b.save()
+
     try:
         rev = branch.revisions.get(commitid=data['commitid'])
     except Revision.DoesNotExist:
@@ -847,8 +924,8 @@ def save_result(data):
             try:
                 saverevisioninfo(rev)
             except RuntimeError as e:
-                logging.warning("unable to save revision %s info: %s", rev, e,
-                                exc_info=True)
+                logger.warning("unable to save revision %s info: %s", rev, e,
+                               exc_info=True)
         rev.save()
 
     exe, created = Executable.objects.get_or_create(
@@ -858,9 +935,9 @@ def save_result(data):
 
     try:
         r = Result.objects.get(
-            revision=rev,executable=exe,benchmark=b,environment=env)
+            revision=rev, executable=exe, benchmark=b, environment=env)
     except Result.DoesNotExist:
-        r = Result(revision=rev,executable=exe,benchmark=b,environment=env)
+        r = Result(revision=rev, executable=exe, benchmark=b, environment=env)
 
     r.value = data["result_value"]
     if 'result_date' in data:
@@ -880,6 +957,7 @@ def save_result(data):
     return (rev, exe, env), False
 
 
+@csrf_exempt
 def add_result(request):
     if request.method != 'POST':
         return HttpResponseNotAllowed('POST')
@@ -887,40 +965,41 @@ def add_result(request):
 
     response, error = save_result(data)
     if error:
-        logging.error("Could not save result: " + response)
+        logger.error("Could not save result: " + response)
         return HttpResponseBadRequest(response)
     else:
         create_report_if_enough_data(response[0], response[1], response[2])
-        logging.debug("add_result: completed")
+        logger.debug("add_result: completed")
         return HttpResponse("Result data saved successfully", status=202)
 
 
+@csrf_exempt
 def add_json_results(request):
     if request.method != 'POST':
         return HttpResponseNotAllowed('POST')
     if not request.POST.get('json'):
         return HttpResponseBadRequest("No key 'json' in POST payload")
     data = json.loads(request.POST['json'])
-    logging.info("add_json_results request with %d entries." % len(data))
+    logger.info("add_json_results request with %d entries." % len(data))
 
     unique_reports = set()
     i = 0
     for result in data:
         i += 1
-        logging.debug("add_json_results: save item %d." % i)
+        logger.debug("add_json_results: save item %d." % i)
         response, error = save_result(result)
         if error:
-            logging.debug(
+            logger.debug(
                 "add_json_results: could not save item %d because %s" % (
                 i, response))
             return HttpResponseBadRequest(response)
         else:
             unique_reports.add(response)
 
-    logging.debug("add_json_results: about to create reports")
+    logger.debug("add_json_results: about to create reports")
     for rep in unique_reports:
         create_report_if_enough_data(rep[0], rep[1], rep[2])
 
-    logging.debug("add_json_results: completed")
+    logger.debug("add_json_results: completed")
 
     return HttpResponse("All result data saved successfully", status=202)
